/*
*	File:		MelodyExtraction.h
*	
*	Version:	1.0
* 
*	Created:	10/08/10
*	
*	Copyright:  Copyright  2010 Angel Noguera, All Rights Reserved
* 
*	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc. ("Apple") in 
*				consideration of your agreement to the following terms, and your use, installation, modification 
*				or redistribution of this Apple software constitutes acceptance of these terms.  If you do 
*				not agree with these terms, please do not use, install, modify or redistribute this Apple 
*				software.
*
*				In consideration of your agreement to abide by the following terms, and subject to these terms, 
*				Apple grants you a personal, non-exclusive license, under Apple's copyrights in this 
*				original Apple software (the "Apple Software"), to use, reproduce, modify and redistribute the 
*				Apple Software, with or without modifications, in source and/or binary forms; provided that if you 
*				redistribute the Apple Software in its entirety and without modifications, you must retain this 
*				notice and the following text and disclaimers in all such redistributions of the Apple Software. 
*				Neither the name, trademarks, service marks or logos of Apple Computer, Inc. may be used to 
*				endorse or promote products derived from the Apple Software without specific prior written 
*				permission from Apple.  Except as expressly stated in this notice, no other rights or 
*				licenses, express or implied, are granted by Apple herein, including but not limited to any 
*				patent rights that may be infringed by your derivative works or by other works in which the 
*				Apple Software may be incorporated.
*
*				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO WARRANTIES, EXPRESS OR 
*				IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY 
*				AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE 
*				OR IN COMBINATION WITH YOUR PRODUCTS.
*
*				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL 
*				DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*				OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, 
*				REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER 
*				UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN 
*				IF APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/
#include "AUEffectBase.h"
#include "MelodyExtractionVersion.h"

#if AU_DEBUG_DISPATCHER
	#include "AUDebugDispatcher.h"
#endif


#ifndef __MelodyExtraction_h__
#define __MelodyExtraction_h__

#pragma mark ____MelodyExtraction Parameters

	static CFStringRef kParamName_Melody_Frec_Min = CFSTR ("Frec Fund Min"); 
	static const float kDefaultValue_Melody_Frec_Min = 20; 
	static const float kMinimumValue_Melody_Frec_Min = 20; 
	static const float kMaximumValue_Melody_Frec_Min = 4500;

	static CFStringRef kParamName_Melody_Frec_Max = CFSTR ("Frec Fund Max"); 
	static const float kDefaultValue_Melody_Frec_Max = 4500; 
	static const float kMinimumValue_Melody_Frec_Max = 20; 
	static const float kMaximumValue_Melody_Frec_Max = 4500;  
	
	static CFStringRef kParamName_Melody_Umbral = CFSTR ("Umbral RMS"); 
	static const float kDefaultValue_Melody_Umbral = 0; 
	static const float kMinimumValue_Melody_Umbral = 0; 
	static const float kMaximumValue_Melody_Umbral = 100; 
	
	static CFStringRef kParamName_Melody_Sol = CFSTR ("Solapamiento"); 
	static const float kDefaultValue_Melody_Sol = 0; 
	static const float kMinimumValue_Melody_Sol = 0; 
	static const float kMaximumValue_Melody_Sol = 100;

	enum 
	{
		kParameter_Frec_Min = 0, 
		kParameter_Frec_Max = 1, 
		kParameter_Umbral = 2,
		kParameter_Sol = 3,
		kNumberOfParameters = 4
	};

#pragma mark ____Melody Extraction Factory Preset Constants 
	//declaracion y definicion de los preajustes de fabrica 
	static const float kParameter_Preset_Frec_Max_Saxo = 720; 
	static const float kParameter_Preset_Frec_Max_Trompeta = 1000;
	static const float kParameter_Preset_Frec_Max_Viola = 1200; 
	static const float kParameter_Preset_Frec_Min_Saxo = 150; 
	static const float kParameter_Preset_Frec_Min_Trompeta = 120;
	static const float kParameter_Preset_Frec_Min_Viola = 100; 
	static const float kParameter_Preset_Umbral_Saxo = 5;
	static const float kParameter_Preset_Umbral_Trompeta = 5;
	static const float kParameter_Preset_Umbral_Viola = 5;
	static const float kParameter_Preset_Sol_Saxo = 50;
	static const float kParameter_Preset_Sol_Trompeta = 50;
	static const float kParameter_Preset_Sol_Viola = 50;


enum { 
kPreset_Saxo = 0, // 7 
kPreset_Trompeta = 1, // 8 
kPreset_Viola = 2,
kNumberPresets = 3 // 9 
}; 

static AUPreset kPresets [kNumberPresets] = { 
{kPreset_Saxo, CFSTR ("Saxo Alto")}, 
{kPreset_Trompeta, CFSTR ("Trompeta")}, 
{kPreset_Viola, CFSTR ("Viola")} 
}; 
static const int kPreset_Default = kPreset_Saxo; 

typedef struct
{
	int pos;
	Float32 mod;
	Float32 phas;
	Float32 omega;
}Partial;

typedef struct
{
	int size;
	Float32 mod[8192];
	Float32 pha[8192];
	Float32 win[8192];
}FFT;

#pragma mark ____MelodyExtraction
class MelodyExtraction : public AUEffectBase
{
public:
	MelodyExtraction(AudioUnit component);
#if AU_DEBUG_DISPATCHER
	virtual ~MelodyExtraction () { delete mDebugDispatcher; }
#endif
	
	virtual AUKernelBase *		NewKernel() { return new MelodyExtractionKernel(this); }
	
	virtual	ComponentResult		GetParameterValueStrings(AudioUnitScope			inScope,
														 AudioUnitParameterID		inParameterID,
														 CFArrayRef *			outStrings);
    
	virtual	ComponentResult		GetParameterInfo(AudioUnitScope			inScope,
												 AudioUnitParameterID	inParameterID,
												 AudioUnitParameterInfo	&outParameterInfo);
    
	virtual ComponentResult		GetPropertyInfo(AudioUnitPropertyID		inID,
												AudioUnitScope			inScope,
												AudioUnitElement		inElement,
												UInt32 &			outDataSize,
												Boolean	&			outWritable );
	
	virtual ComponentResult		GetProperty(AudioUnitPropertyID inID,
											AudioUnitScope 		inScope,
											AudioUnitElement 		inElement,
											void *			outData);
	
 	virtual	bool				SupportsTail () { return true; }
	
	/*! @method Version */
	virtual ComponentResult	Version() { return kMelodyExtractionVersion; }
	
    virtual ComponentResult GetPresets ( 
		CFArrayRef *outData 
	) const; 

	virtual OSStatus NewFactoryPresetSet ( 
		const AUPreset &inNewFactoryPreset 
	); 
	
protected:
		class MelodyExtractionKernel : public AUKernelBase		
	{
public:
		MelodyExtractionKernel(AUEffectBase *inAudioUnit )
		: AUKernelBase(inAudioUnit)
	{
	}
		
        virtual void 	Process(	const Float32 	*inSourceP,
										Float32		 	*inDestP,
										UInt32 			inFramesToProcess,
										UInt32			inNumChannels,
										bool			&ioSilence);
		
        virtual void	Reset();
		virtual int		FreqToPos(Float32 freq, Float32 freqRes);
		virtual Float32	PosToFreq(int pos, Float32 freqRes);
		virtual	Float32	RMS(Float32 array[], int size);
		virtual void	HPS(int winSize, int partNum) ;
		virtual int		MaximumSearch(int begOfRange, int endOfRange);
		virtual void	Synthesis(UInt32 nSample, int partInd);
		virtual int		SelectPeakCloseTo(Float32 spectrum[], int tamwindow, int sr, Float32 centerfreq, int tolerance) ;
		virtual	void	Overlap(int size, Float32 coef);
		
		private: 
	
		Float32 in[8192], averageSpectrum[4096], synthSignal[4096], preSynthSignal[4096], overlapSignal[4096];
		FFT inSpectrum;
		Partial part[16], prePart[16];
		int peaks[4096], guard;
	
	};
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


#endif