#ifndef PARAMS
#define PARAMS
#include <math.h>

/***************** Spectrum analysis parameters *****************/

#define WINDOW		0  	// 0: Hanning, 1: Blackman-Harris 92dB
#define kDR 	 	1       // Decimation ratio applied
#define kPERCENTAGE	50	// Overlapping percentage
#define kN		1	// Input window size (0->43ms, 1->96ms, 2->192ms)
#define kRES	 	4       // winsize=winsize*res (Zero padding) for 44kHz. If sr=22kHz, this value is multiplied by two
#define DIVFACTOR	2147483000.0	// 2^16 * 2^15= 2^21 Divide input (time-signal) between this value before FFT computation
#define INTERPOLATION   false	// Quadratic interpolation

#define MFT		false	// Multiresolution fourier transform
#define SIZEMFTFREQS	2	// Size of mftfreqs
const double mftfreqs[]= {150,2000};	// Limit frequencies for MFT

/***************** Sinusoidal likeness parameters ***************/

#define kSINUSOIDAL	false   // If true, sinusoidal peak picking is activated. If false, normal spectrum peak piking will be computed 

#define FSIN1		900	// Limit frequencies for sinusoidal likeness
#define FSIN2		1800
#define FSIN3		3600

#define kW1		20.0	// Bandwidth (Hz) for sinusoidal likeness
#define kW2		50.0
#define kW3		100.0
#define kW4		200.0

#define kMINSINLIKEVAL   0.1	// 0.1 Minimum sinusoidal likeness value for considering peaks. Not used if kSINUSOIDAL=false 
				// when spectral whitening is activated, this value should be about 0.0001. Otherwise, about 0.1

/******************* Spectral whitening  **************************/

#define SPECTRALWHITENING 	false
#define COMPRESSIONCOEF		0.33

/****************** Processing parameters ************************/

#define kMINPEAKVALUE   0.1 	// 0.4 Minimum frequency spectrum value for considering as a peak.
#define AVERAGE		1   	// Average spectra. With AVERAGE==1 the results are always better (better use it)
#define MAXPOLYPHONY    6       // Max number of notes detected in each frame
#define MAXHARMONICS   10 	// Max number of harmonics (for efficiency)
#define MAXCANDIDATES  10	// Max number of f0 candidates
#define MINHARMONICS   	2	// Minimum number of harmonics for a f0 candidate
#define STABLEFRAMES	1       // Peak picking is done n frames after onset (if possible) when -onset is used

#define kSHARPNESS	2.0	// 
#define kLOUDNESS	0.1  	// Minimum note loudness relative to the maximum note in surrounding area
#define kMINLOUDNESS	5  	// Minimum loudness for a note
#define kDISCARDH	0.00	// If 0.1, harmonics lower to 10% of maximum spectral pattern value will be discarded
#define LOWESTNOTE	38.0	// Notes lower than this threshold (Hz) will not be considered
#define HIGHESTNOTE	2100.0	// Notes higher than this threshold (Hz) will not be considered

//#define CENTERF0INPITCH	   false 	// Ayuda muchsimo para las bajas frecuencias, cuando zeropad es bajo (2).
#define F0CANDIDATESCLOSETOPITCH   true	// To discard candidates that are not close to the western fundamental frequencies

#define INHARMFACTOR	0.0 	// NOT USED
#define PEAKLIMIT       11  	// If 0, the high and low limits to select a peak given a center frequency are relative to stepinterharm variable; otherwise, will be +-PEAKLIMIT Hz  

/***************** Parameters for onset detection ****************/

#define CONTEXT		0	// Number of context frames for onset detection
#define kLSM		1.0 	// Silence threshold for onset detection
#define NEWPAR		0.18	// Threshold at the end of onset detection stage

/*************** Perceptual weighting (A-weighting curve) *********/

#define ProcessWeighting false  // Perceptual weighting (A-weighting curve)

/*************** Bands parameters **********************************/

#define kMINBANDFREQ	48.9994087219		// MidiNote= G0 (1st center frequency of 1/12 bands)
#define MINBANDDISTANCE 15	// Only f0 candidates closer than MINBANDDISTANCE Hz to a center frequency band will be considered

const double stepinterbands=pow(2.0,1.0/24.0);	// Bands separation (Hz)
const double candrange=pow(2.0,1.0/24.0);	// Allows only f0 candidates in a range close to freqrange from its nearest band
const double stepinterharm=pow(2.0,1.0/24.0); 	//1/96: 1/4 semitone

/*********************** Gaussian smoothness **************/

const int tamgaussian=3;		      // Gaussian window size
static const double gaussian[]={0.2,0.6,0.2}; // Gaussian window values

/*********************** Frame by frame smoothing **************/
#define kMINDUR		0	// Number of frames to clean non consecutive notes using cleannotes

#endif
